﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.Web.Http;
using Polly;
using Polly.Retry;

namespace PollyOnRetry.Controllers
{
    [RoutePrefix("api/values")]
    public class ValuesController : ApiController
    {
        readonly RetryPolicy<HttpResponseMessage> _httpRequestPolicy;
        private HttpClient _httpClient;
        public ValuesController()
        {
            _httpRequestPolicy = Policy.HandleResult<HttpResponseMessage>(
                    r => r.StatusCode == HttpStatusCode.InternalServerError ||
                         r.StatusCode == HttpStatusCode.RequestTimeout || r.StatusCode == HttpStatusCode.Unauthorized)
                .WaitAndRetryAsync(3, retryAttempt => TimeSpan.FromSeconds(retryAttempt), (response, timespan) =>
                {
                    if (response.Result.StatusCode == HttpStatusCode.Unauthorized)
                    {
                        PerformReauthorization();
                    }
                });
        }

        public async Task<IHttpActionResult> Get()
        {
            _httpClient = GetHttpClient("BadAuthCode"); // start with a bad auth token

            string requestEndpoint = $"numbers/Get";

            HttpResponseMessage httpResponse = await _httpRequestPolicy.ExecuteAsync(() => _httpClient.GetAsync(requestEndpoint));

            IEnumerable<int> numbers = await httpResponse.Content.ReadAsAsync<IEnumerable<int>>();

            return Ok(numbers);
        }

        private void PerformReauthorization()
        {
            // here you would get a new token, call some other service, etc.
            _httpClient = GetHttpClient("GoodAuthCode"); // pass in the good auth token
        }

        private HttpClient GetHttpClient(string authCookieValue)
        {
            var cookieContainer = new CookieContainer();
            var handler = new HttpClientHandler() {CookieContainer = cookieContainer};
            cookieContainer.Add(new Uri("http://localhost"),new Cookie("Auth", authCookieValue));
            _httpClient = new HttpClient(handler);
            _httpClient.BaseAddress = new Uri(@"http://localhost:2629/api/");

            _httpClient.DefaultRequestHeaders.Accept.Clear();
            _httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            return _httpClient;
        }
    }
}
